@@ -152,7 +152,7 @@ MainWindow::MainWindow(Player *player, QWidget *parent) :
     connect(openDialog, SIGNAL(fileSelected(QString)), this, SLOT(setSongPath(QString)));
     connect(ui->actionFileSave, SIGNAL(triggered()), this, SLOT(save()));
     connect(ui->actionFileSaveAs, SIGNAL(triggered()), this, SLOT(saveAs()));
-    connect(ui->actionFileQuit, SIGNAL(triggered()), qApp, SLOT(quit()));
+    connect(ui->actionFileQuit, SIGNAL(triggered()), this, SLOT(quit()));
     connect(ui->actionEditCut, SIGNAL(triggered()), this, SLOT(cutSelection()));
     connect(ui->actionEditCopy, SIGNAL(triggered()), this, SLOT(copySelection()));
     connect(ui->actionEditPaste, SIGNAL(triggered()), this, SLOT(pasteSelection()));
@@ -301,9 +301,9 @@ bool MainWindow::eventFilter(QObject *watched, QEvent *event)
 void MainWindow::wheelEvent(QWheelEvent *event)
 {
     if (event->delta() > 0) {
-        player->setLine(player->line() - 8);
+        player->setLine(player->line() - 4);
     } else if (event->delta() < 0) {
-        player->setLine(player->line() + 8);
+        player->setLine(player->line() + 4);
     }
 }
 
@@ -316,6 +316,10 @@ void MainWindow::mousePressEvent(QMouseEvent *)
 
 bool MainWindow::keyPress(QKeyEvent *event)
 {
+    if (ui->menubar->activeAction() != NULL) {
+        return false;
+    }
+
     Block *block = ui->tracker->block();
     bool shift = (event->modifiers() & Qt::ShiftModifier) != 0;
     bool ctrl = (event->modifiers() & Qt::ControlModifier) != 0;
@@ -840,6 +844,8 @@ void MainWindow::setSong(Song *song)
 {
     if (this->song != NULL) {
         disconnect(ui->actionSettingsSendMidiSync, SIGNAL(triggered(bool)), this->song, SLOT(setSendSync(bool)));
+        disconnect(this->song, SIGNAL(nameChanged()), this, SLOT(setWindowTitle()));
+        disconnect(this->song, SIGNAL(modifiedChanged()), this, SLOT(setWindowTitle()));
     }
 
     this->song = song;
@@ -853,12 +859,15 @@ void MainWindow::setSong(Song *song)
     setTime(0);
     setInstrument(ui->spinBoxInstrument->value());
     ui->tracker->setLine(player->line());
+    setWindowTitle();
 
     instrumentPropertiesDialog->setSong(song);
     instrumentPropertiesDialog->setInstrument(ui->spinBoxInstrument->value());
     ui->actionSettingsSendMidiSync->setChecked(song->sendSync());
 
     connect(ui->actionSettingsSendMidiSync, SIGNAL(triggered(bool)), song, SLOT(setSendSync(bool)));
+    connect(this->song, SIGNAL(nameChanged()), this, SLOT(setWindowTitle()));
+    connect(this->song, SIGNAL(modifiedChanged()), this, SLOT(setWindowTitle()));
 }
 
 void MainWindow::setSection(unsigned int section)
@@ -870,13 +879,13 @@ void MainWindow::setPlayseq(unsigned int playseq)
 {
     if (this->playseq < song->playseqs()) {
         disconnect(song->playseq(this->playseq), SIGNAL(nameChanged(QString)), this, SLOT(setPlayseq()));
-        disconnect(song->playseq(this->playseq), SIGNAL(playseqChanged()), this, SLOT(setPosition()));
+        disconnect(song->playseq(this->playseq), SIGNAL(lengthChanged()), this, SLOT(setPosition()));
     }
 
     this->playseq = playseq;
 
     connect(song->playseq(playseq), SIGNAL(nameChanged(QString)), this, SLOT(setPlayseq()));
-    connect(song->playseq(playseq), SIGNAL(playseqChanged()), this, SLOT(setPosition()));
+    connect(song->playseq(playseq), SIGNAL(lengthChanged()), this, SLOT(setPosition()));
 
     QString name = song->playseq(playseq)->name();
     if (name.isEmpty()) {
@@ -973,7 +982,7 @@ void MainWindow::setSelection(int startTrack, int startLine, int endTrack, int e
 
 void MainWindow::showAbout()
 {
-    QMessageBox::about(this, tr("About Tutka"), tr("Tutka 1.0.1\n\n© 2002-2014 Vesa Halttunen <vesuri@jormas.com>"));
+    QMessageBox::about(this, tr("About Tutka"), tr("Tutka 1.0.2\n\n© 2002-2014 Vesa Halttunen <vesuri@jormas.com>"));
 }
 
 void MainWindow::cutSelection()
@@ -1267,3 +1276,35 @@ void MainWindow::setGeometryFromString(QWidget *widget, const QString &string)
         }
     }
 }
+
+void MainWindow::setWindowTitle()
+{
+    QMainWindow::setWindowTitle(tr("Tutka: %1%2").arg(song->name()).arg(song->isModified() ? " (*)" : ""));
+}
+
+void MainWindow::quit()
+{
+    if (song != NULL && song->isModified()) {
+        QMessageBox messageBox;
+        messageBox.setWindowTitle(tr("Tutka"));
+        messageBox.setText(tr("The song has been modified."));
+        messageBox.setInformativeText(tr("Do you want to save your changes?"));
+        messageBox.setStandardButtons(QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
+        messageBox.setDefaultButton(QMessageBox::Save);
+        int ret = messageBox.exec();
+
+        switch (ret) {
+        case QMessageBox::Save:
+            save();
+            qApp->quit();
+            break;
+        case QMessageBox::Discard:
+            qApp->quit();
+            break;
+        default:
+            break;
+        }
+    } else {
+        qApp->quit();
+    }
+}
